<?php
// +-------------------------------------------------+
// | 2002-2007 PMB Services / www.sigb.net pmb@sigb.net et contributeurs (voir www.sigb.net)
// +-------------------------------------------------+
// $Id: mail_configuration.class.php,v 1.1.2.2 2022/09/08 08:43:28 dgoron Exp $

if (stristr($_SERVER['REQUEST_URI'], ".class.php")) die("no access");

class mail_configuration {
    
	protected $id;
	
	protected $type;
	
	protected $name;
	
	protected $protocol;
	
	protected $hote;
	
	protected $port;
	
	protected $authentification;
	
	protected $user;
	
	protected $password;
	
	protected $secure_protocol;
	
	protected $authentification_type;
	
	protected $allowed_hote_override;
	
	protected $allowed_authentification_override;
	
	protected $confidential;
	
	protected $domain;
	
	protected $domain_name;
	
	public function __construct($name='') {
		$this->id = 0;
		$this->name = $name;
		if(strpos($this->name, '@') !== false) {
			$this->type = 'address';
		} else {
			$this->type = 'domain';
		}
		$this->fetch_data();
	}
	
	protected function init_properties() {
		if(strpos($this->name, '@') !== false) {
			$this->domain_name = substr($this->name, strpos($this->name, '@')+1);
		} else {
			$this->domain_name = $this->name;
		}
		$this->protocol = 'SMTP';
		$this->hote = '';
		$this->port = 25;
		$this->authentification = 0;
		$this->user = '';
		$this->password = '';
		$this->secure_protocol = '';
		$this->authentification_type = '';
		$this->allowed_hote_override = 0;
		$this->allowed_authentification_override = 0;
	}
	
	protected function fetch_object($row) {
		$this->protocol = $row->mail_configuration_protocol;
		$this->hote = $row->mail_configuration_hote;
		$this->port = $row->mail_configuration_port;
		$this->authentification = $row->mail_configuration_authentication;
		$this->user = $row->mail_configuration_user;
		$this->password = convert_uudecode($row->mail_configuration_password);
		$this->secure_protocol = $row->mail_configuration_secure_protocol;
		$this->authentification_type = $row->mail_configuration_authentification_type;
		$this->allowed_hote_override = $row->mail_configuration_allowed_hote_override;
		$this->allowed_authentification_override = $row->mail_configuration_allowed_authentification_override;
	}
	
	protected function fetch_data() {
		$this->init_properties();
		if($this->is_confidential()) {
			pmb_error::get_instance(static::class)->add_message("permission_denied", "mail_configuration_action_edit_locked");
		}
		if($this->name) {
			$query = "SELECT * FROM mails_configuration WHERE name_mail_configuration = '".addslashes($this->name)."'";
			$result = pmb_mysql_query($query);
			if(pmb_mysql_num_rows($result)) {
				$row = pmb_mysql_fetch_object($result);
				$this->fetch_object($row);
			} elseif($this->type == 'address' && $this->get_domain()->get_name()) {
				$query = "SELECT * FROM mails_configuration WHERE name_mail_configuration = '".addslashes($this->get_domain()->get_name())."'";
				$result = pmb_mysql_query($query);
				if(pmb_mysql_num_rows($result)) {
					$row = pmb_mysql_fetch_object($result);
					$this->fetch_object($row);
				}
			}
		}
	}
	
	protected function get_protocol_selector() {
		return "
			<select name='mail_configuration_protocol' ".($this->is_hote_readonly() ? "readonly='readonly' disabled='disabled'" : "").">
					<option value='SMTP' ".($this->protocol == 'SMTP' ? "selected='selected'" : "").">SMTP</option>
			</select>";	
	}
	
	protected function get_secure_protocol_selector() {
		return "
			<select name='mail_configuration_secure_protocol' ".($this->is_authentification_readonly() ? "readonly='readonly' disabled='disabled'" : "").">
					<option value='' ".(empty($this->secure_protocol) ? "selected='selected'" : "")."></option>
					<option value='starttls' ".($this->secure_protocol == 'starttls' ? "selected='selected'" : "").">STARTTLS</option>
					<option value='ssl' ".($this->secure_protocol == 'ssl' ? "selected='selected'" : "").">SSL</option>
					<option value='tls' ".($this->secure_protocol == 'tls' ? "selected='selected'" : "").">TLS</option>
			</select>";
	}
	
	protected function get_authentification_type_selector() {
		$selector = "<select name='mail_configuration_authentification_type' ".($this->is_authentification_readonly() ? "readonly='readonly' disabled='disabled'" : "").">
						<option value='' ".(empty($this->authentification_type) ? "selected='selected'" : "")."></option>";
		foreach (mail::SMTP_AUTH_TYPES as $auth_type) {
			$selector .= "<option value='".$auth_type."' ".($this->authentification_type == $auth_type ? "selected='selected'" : "").">".$auth_type."</option>";
		}
		$selector .= "</select>";
		
		return $selector;
	}
	
	public function get_form() {
		global $msg, $charset;
		global $mail_configuration_address_content_form, $mail_configuration_domain_content_form;
		global $mail_configuration_hote_content_form, $mail_configuration_authentification_content_form;
		global $mail_configuration_allowed_overrides_content_form;
		
		$content_form = '';
		switch ($this->type) {
			case 'domain':
				$content_form .= $mail_configuration_domain_content_form;
				$content_form .= $mail_configuration_hote_content_form;
				$content_form .= $mail_configuration_authentification_content_form;
				$content_form .= $mail_configuration_allowed_overrides_content_form;
				break;
			case 'address':
				$content_form .= $mail_configuration_address_content_form;
				$content_form .= $mail_configuration_hote_content_form;
				$content_form .= $mail_configuration_authentification_content_form;
				break;
		}
		$interface_form = new interface_form('mail_configuration_form');
		$interface_form->set_label($msg['mail_configuration_edit']);
		$content_form = str_replace('!!type!!', htmlentities($this->type, ENT_QUOTES, $charset), $content_form);
		$content_form = str_replace('!!name!!', htmlentities($this->name, ENT_QUOTES, $charset), $content_form);
// 		$content_form = str_replace('!!protocol_selector!!', $this->get_protocol_selector(), $content_form);
		$content_form = str_replace('!!hote!!', htmlentities($this->hote, ENT_QUOTES, $charset), $content_form);
		$content_form = str_replace('!!port!!', $this->port, $content_form);
		$content_form = str_replace('!!authentification!!', ($this->authentification ? "checked='checked'" : ""), $content_form);
		$content_form = str_replace('!!user!!', htmlentities($this->user, ENT_QUOTES, $charset), $content_form);
		$content_form = str_replace('!!password!!', htmlentities($this->password, ENT_QUOTES, $charset), $content_form);
		$content_form = str_replace('!!secure_protocol_selector!!', $this->get_secure_protocol_selector(), $content_form);
		$content_form = str_replace('!!authentification_type_selector!!', $this->get_authentification_type_selector(), $content_form);
		
		$content_form = str_replace('!!allowed_hote_override!!', ($this->get_domain()->is_allowed_hote_override() ? "checked='checked'" : ""), $content_form);
		$content_form = str_replace('!!allowed_authentification_override!!', ($this->get_domain()->is_allowed_authentification_override() ? "checked='checked'" : ""), $content_form);
		
		$content_form = str_replace('!!hote_readonly!!', ($this->is_hote_readonly() ? "readonly='readonly' disabled='disabled'" : ""), $content_form);
		$content_form = str_replace('!!authentification_readonly!!', ($this->is_authentification_readonly() ? "readonly='readonly' disabled='disabled'" : ""), $content_form);
		
		$interface_form->set_object_id(($this->is_in_database() ? 1 : 0))
		->set_confirm_delete_msg($msg['confirm_suppr_de']." ".$this->name." ?")
		->set_content_form($content_form)
		->set_table_name('mails_configuration')
		->set_field_focus('mail_configuration_hote');
		return $interface_form->get_display();
	}
	
	public function set_properties_from_form() {
		global $mail_configuration_hote, $mail_configuration_port;
		global $mail_configuration_authentification, $mail_configuration_user, $mail_configuration_password;
		global $mail_configuration_secure_protocol, $mail_configuration_authentification_type;
		global $mail_configuration_allowed_hote_override, $mail_configuration_allowed_authentification_override;
		
		switch ($this->type) {
			case 'domain':
				$this->hote = stripslashes($mail_configuration_hote);
				$this->port = intval($mail_configuration_port);
				$this->authentification = intval($mail_configuration_authentification);
				$this->user = stripslashes($mail_configuration_user);
				$this->password = stripslashes($mail_configuration_password);
				$this->secure_protocol = stripslashes($mail_configuration_secure_protocol);
				$this->authentification_type = stripslashes($mail_configuration_authentification_type);
				$this->allowed_hote_override = intval($mail_configuration_allowed_hote_override);
				$this->allowed_authentification_override = intval($mail_configuration_allowed_authentification_override);
				break;
			case 'address':
				if($this->get_domain()->is_allowed_hote_override()) {
					$this->hote = stripslashes($mail_configuration_hote);
					$this->port = intval($mail_configuration_port);
				} else {
					$this->hote = '';
					$this->port = '';
				}
				if($this->get_domain()->is_allowed_authentification_override()) {
					$this->authentification = intval($mail_configuration_authentification);
					$this->user = stripslashes($mail_configuration_user);
					$this->password = stripslashes($mail_configuration_password);
					$this->secure_protocol = stripslashes($mail_configuration_secure_protocol);
					$this->authentification_type = stripslashes($mail_configuration_authentification_type);
				} else {
					$this->authentification = 0;
					$this->user = '';
					$this->password = '';
					$this->secure_protocol = '';
					$this->authentification_type = '';
				}
				$this->allowed_hote_override = 1;
				$this->allowed_authentification_override = 1;
				break;
		}
	}
	
	public function is_in_database() {
		$query = 'SELECT * FROM mails_configuration
			WHERE name_mail_configuration = "'.addslashes($this->name).'"';
		$result = pmb_mysql_query($query);
		if(pmb_mysql_num_rows($result)) {
			return true;
		}
		return false;
	}
	
	public function save() {
		if($this->is_in_database()){
			$query = "update mails_configuration set ";
			$clause = " where name_mail_configuration = '".addslashes($this->name)."'";
		}else{
			$query = "insert into mails_configuration set 
				mail_configuration_type = '".addslashes($this->type)."',
				name_mail_configuration = '".addslashes($this->name)."',
			";
			$clause= "";
		}
		$query.= "
			mail_configuration_protocol = '".addslashes($this->protocol)."',
			mail_configuration_hote = '".addslashes($this->hote)."',
			mail_configuration_port = '".$this->port."',
			mail_configuration_authentication = '".$this->authentification."',
			mail_configuration_user = '".addslashes($this->user)."',
			mail_configuration_password = '".addslashes(convert_uuencode($this->password))."',
			mail_configuration_secure_protocol = '".addslashes($this->secure_protocol)."',
			mail_configuration_authentification_type = '".addslashes($this->authentification_type)."',
			mail_configuration_allowed_hote_override = '".intval($this->allowed_hote_override)."',
			mail_configuration_allowed_authentification_override = '".intval($this->allowed_authentification_override)."'
			".$clause;
		
		$result = pmb_mysql_query($query);
		if($result){
			return true;
		}
		return false;
	}
	
	public static function delete($name) {
		$query = "delete from mails_configuration where name_mail_configuration = '".addslashes($name)."'";
		pmb_mysql_query($query);
		return true;
	}
	
	public function get_id() {
		return $this->name;
	}
	
	public function get_type() {
		return $this->type;
	}
	
	public function get_name() {
		return $this->name;
	}
	
	public function get_domain() {
		if(!isset($this->domain)) {
			$this->domain = new mail_configuration($this->domain_name);
		}
		return $this->domain;
	}
	
	public function get_domain_name() {
		return $this->domain_name;
	}
	
	public function get_protocol() {
		return $this->protocol;
	}
	
	public function get_hote() {
		return $this->hote;
	}
	
	public function get_port() {
		return $this->port;
	}
	
	public function get_authentification() {
		return $this->authentification;
	}
	
	public function get_user() {
		return $this->user;
	}
	
	public function get_password() {
		return $this->password;
	}
	
	public function get_secure_protocol() {
		return $this->secure_protocol;
	}
	
	public function get_authentification_type() {
		return $this->authentification_type;
	}
	
	public function set_type($type) {
		$this->type = $type;
		return $this;
	}
	
	public function set_name($name) {
		$this->name = $name;
		if(empty($this->type)) {
			if(strpos($this->name, '@') !== false) {
				$this->type = 'address';
			} else {
				$this->type = 'domain';
			}
		}
		return $this;
	}
	
	public function set_protocol($protocol) {
		$this->protocol = $protocol;
		return $this;
	}
	
	public function set_hote($hote) {
		$this->hote = $hote;
		return $this;
	}
	
	public function set_port($port) {
		$this->port = $port;
		return $this;
	}
	
	public function set_authentification($authentification) {
		$this->authentification = $authentification;
		return $this;
	}
	
	public function set_user($user) {
		$this->user = $user;
		return $this;
	}
	
	public function set_password($password) {
		$this->password = $password;
		return $this;
	}
	
	public function set_secure_protocol($secure_protocol) {
		$this->secure_protocol = $secure_protocol;
		return $this;
	}
	
	public function set_authentification_type($authentification_type) {
		$this->authentification_type = $authentification_type;
		return $this;
	}
	
	public function is_allowed_hote_override() {
		return $this->allowed_hote_override;
	}
	
	public function is_allowed_authentification_override() {
		return $this->allowed_authentification_override;
	}
	
	public function set_confidential($confidential) {
		$this->confidential = intval($confidential);
		return $this;
	}
	
	public function is_confidential() {
		global $pmb_mail_adresse_from, $opac_mail_adresse_from, $opac_biblio_email;
		global $PMBuseremail;
		
		if(!isset($this->confidential)) {
			$this->confidential = true;
			switch ($this->type) {
				case 'domain':
					$this->confidential = static::domain_is_confidential($this->name);
					break;
				case 'address':
				default:
					if($pmb_mail_adresse_from == $this->name || $opac_mail_adresse_from == $this->name || $opac_biblio_email == $this->name) {
						$this->confidential = false;
					}
					
					if($PMBuseremail == $this->name) {
						$this->confidential = false;
					}
					$query = "SELECT count(*) as nb FROM docs_location WHERE email = '".$this->name."'";
					$result = pmb_mysql_query($query);
					if(pmb_mysql_result($result, 0, 'nb')) {
						$this->confidential = false;
					}
					$query = "SELECT count(*) as nb FROM coordonnees JOIN entites ON entites.id_entite = coordonnees.num_entite WHERE type_entite = 1 AND email = '".$this->name."'";
					$result = pmb_mysql_query($query);
					if(pmb_mysql_result($result, 0, 'nb')) {
						$this->confidential = false;
					}
					break;
			}
		}
		return $this->confidential;
	}
	
	public function is_hote_readonly() {
		if($this->type == 'address' && empty($this->get_domain()->is_allowed_hote_override())) {
			return true;
		}
		return false;
	}
	
	public function is_authentification_readonly() {
		if($this->type == 'address' && empty($this->get_domain()->is_allowed_authentification_override())) {
			return true;
		}
		return false;
	}
	
	protected static function table_exists() {
		$query = "SHOW TABLES LIKE 'mails_configuration'";
		$result = pmb_mysql_query($query);
		if(pmb_mysql_num_rows($result)) {
			return true;
		}
		return false;
	}
	
	public static function get_address_configuration($address='') {
		if(!static::table_exists()) {
			return array();
		}
		$mail_configuration = new mail_configuration($address);
		if($mail_configuration->is_in_database()) {
			return array(
					strtolower($mail_configuration->get_protocol()),
					$mail_configuration->get_hote().(!empty($mail_configuration->get_port()) ? ":".$mail_configuration->get_port() : ''),
					$mail_configuration->get_authentification(),
					$mail_configuration->get_user(),
					$mail_configuration->get_password(),
					$mail_configuration->get_secure_protocol(),
					$mail_configuration->get_authentification_type()
			);
		} else {
			$mail_configuration = $mail_configuration->get_domain();
			if($mail_configuration->is_in_database()) {
				return array(
						strtolower($mail_configuration->get_protocol()),
						$mail_configuration->get_hote().(!empty($mail_configuration->get_port()) ? ":".$mail_configuration->get_port() : ''),
						$mail_configuration->get_authentification(),
						$mail_configuration->get_user(),
						$mail_configuration->get_password(),
						$mail_configuration->get_secure_protocol(),
						$mail_configuration->get_authentification_type()
				);
			}
		}
		return array();
	}
	
	public static function domain_is_confidential($domain) {
		global $pmb_mail_adresse_from, $opac_mail_adresse_from, $opac_biblio_email;
		global $PMBuseremail;
		
		$confidential = true;
		if(strpos($pmb_mail_adresse_from, $domain) !== false || strpos($opac_mail_adresse_from, $domain) !== false || strpos($opac_biblio_email, $domain) !== false) {
			$confidential = false;
		}
		if(strpos($PMBuseremail, $domain) !== false) {
			$confidential = false;
		}
		$query = "SELECT count(*) as nb FROM docs_location WHERE email LIKE '%".addslashes($domain)."'";
		$result = pmb_mysql_query($query);
		if(pmb_mysql_result($result, 0, 'nb')) {
			$confidential = false;
		}
		$query = "SELECT count(*) as nb FROM coordonnees JOIN entites ON entites.id_entite = coordonnees.num_entite WHERE type_entite = 1 AND email LIKE '%".addslashes($domain)."'";
		$result = pmb_mysql_query($query);
		if(pmb_mysql_result($result, 0, 'nb')) {
			$confidential = false;
		}
		return $confidential;
	}
}
	
